@@ -1,6 +1,5 @@
 import dataclasses
 import enum
-import logging
 from typing import Optional, Tuple
 from urllib import parse
 from urllib.parse import parse_qs, urlparse
@@ -14,7 +13,7 @@ from ngr_spider.constants import (  # type: ignore
     WCS_PROTOCOL,
     WFS_PROTOCOL,
     WMS_PROTOCOL,
-    WMTS_PROTOCOL
+    WMTS_PROTOCOL,
 )
 from ngr_spider.decorators import nested_dataclass
 
@@ -27,6 +26,15 @@ def get_query_param_val(url, param_name):
         return ""
 
 
+class LogLevel(enum.Enum):
+    Error = "ERROR"
+    Info = "INFO"
+    Debug = "DEBUG"
+
+    def __str__(self):
+        return self.value
+
+
 class LayersMode(enum.Enum):
     Flat = "flat"
     Services = "services"
@@ -36,25 +44,11 @@ class LayersMode(enum.Enum):
         return self.value
 
 
-def ExcludeIfNone(value):
-    """Do not include field for None values"""
-    return value is None
-
-
 @dataclasses.dataclass
-class CswListRecord:
-    title: str
-    abstract: str
-    type: str
-    identifier: str
-    keywords: list[str]
-    modified: str
-
-
-@dataclasses.dataclass
-class WmsStyle:
+class Style:
     title: str
     name: str
+    legend_url: str = ""
 
 
 @dataclasses.dataclass
@@ -67,7 +61,7 @@ class Layer:
 
 @dataclasses.dataclass
 class WmsLayer(Layer):
-    styles: list[WmsStyle]
+    styles: list[Style]
     crs: str
     minscale: str = ""
     maxscale: str = ""
@@ -75,6 +69,7 @@ class WmsLayer(Layer):
 
 @dataclasses.dataclass
 class WmtsLayer(Layer):
+    styles: list[Style]
     tilematrixsets: str
     imgformats: str
 
@@ -329,6 +324,7 @@ def list_duplicates(seq):
 class CswServiceRecord(JSONWizard):
     title: str
     abstract: str
+    date_stamp: str
     use_limitation: str
     keywords: list[str]
     operates_on: str
@@ -363,6 +359,10 @@ class CswServiceRecord(JSONWizard):
         )
         return f"{self.__class__.__name__}({dict_repr})"
 
+    def get_date_stamp(self):
+        xpath_query = f".//gmd:dateStamp/gco:Date/text()"
+        return self.get_text_xpath(xpath_query)
+
     def get_record_identifier(self):
         xpath_query = f".//gmd:fileIdentifier/gco:CharacterString/text()"
         return self.get_text_xpath(xpath_query)
@@ -423,7 +423,7 @@ class CswServiceRecord(JSONWizard):
                             keywords_result[keyword_ns] = []
                         keywords_result[keyword_ns].append(keyword_val)
                     except IndexError:
-                        logging.error(
+                        L.error(
                             f"unexpected error while retrieving keyword for record {self.metadata_id}"
                         )
 
@@ -481,6 +481,7 @@ class CswServiceRecord(JSONWizard):
         self.root = etree.fromstring(xml, parser=parser)
         self.metadata_id = self.get_record_identifier()
         self.title = self.get_title()
+        self.date_stamp = self.get_date_stamp()
         self.abstract = self.get_abstract()
         self.use_limitation = self.get_use_limitation()
         # self.point_of_contact = self.get_point_of_contact()
@@ -491,3 +492,27 @@ class CswServiceRecord(JSONWizard):
         self.service_url = self.get_service_url(service_el)
         self.service_protocol = self.get_service_protocol(service_el)
         self.service_description = self.get_service_description(service_el)
+
+        if self.service_protocol in [
+            WMS_PROTOCOL,
+            WFS_PROTOCOL,
+            WCS_PROTOCOL,
+            WMTS_PROTOCOL,
+        ]:
+            service_url = self.service_url.partition("?")[0]
+            protocol = self.service_protocol
+            query_param_svc_type = protocol.split(":")[1]
+            if (
+                "https://geodata.nationaalgeoregister.nl/tiles/service/wmts"
+                in service_url
+            ):  # shorten paths, some wmts services have redundant path elements in service_url
+                service_url = (
+                    "https://geodata.nationaalgeoregister.nl/tiles/service/wmts"
+                )
+            if service_url.endswith(
+                "/WMTSCapabilities.xml"
+            ):  # handle cases for restful wmts url, assume kvp variant is supported
+                service_url = service_url.replace("/WMTSCapabilities.xml", "")
+            self.service_url = (
+                f"{service_url}?request=GetCapabilities&service={query_param_svc_type}"
+            )
